@@ -60,7 +60,7 @@ our @ISA = qw(DiaColloDB::Persistent);
 ##    f12 => \%f12,       ##-- collocation frequencies, %f12 = ($i2=>$f12, ...)
 ##    titles => \@titles, ##-- item group titles (default:undef: unknown)
 ##    #
-##    eps => $eps,       ##-- smoothing constant (default=0.5)
+##    eps => $eps,       ##-- smoothing constant (default=0) #0.5
 ##    score => $func,    ##-- selected scoring function qw(f fm lf lfm mi1 mi3 milf ld ll)
 ##    milf => \%milf_12, ##-- score: mutual information * logFreq a la Wortprofil; requires compile_milf()
 ##    mi1 => \%mi1_12,   ##-- score: mutual information; requires compile_mi1()
@@ -79,7 +79,7 @@ sub new {
 		    f1=>0,
 		    f2=>{},
 		    f12=>{},
-		    eps=>0.5,
+		    eps=>0, #0.5,
 		    #titles=>undef,
 		    #ld=>{},
 		    @_
@@ -324,6 +324,17 @@ sub uncompile {
   return $_[0];
 }
 
+## $prf = $prf->compile_clean(%opts)
+##  + bashes non-finite compiled score values to undef
+sub compile_clean {
+  my $prf = shift;
+  return $prf if (!$prf->{score} || !$prf->{$prf->{score}});
+  foreach (values %{$prf->{$prf->{score}}}) {
+    $_ = undef if (isInf($_));
+  }
+  return $prf;
+}
+
 ## $prf = $prf->compile_f()
 ##  + just sets $prf->{score} = 'f12'
 sub compile_f {
@@ -356,13 +367,13 @@ sub compile_lf {
   my ($prf,%opts) = @_;
   my $pf12 = $prf->{f12};
   my $lf   = $prf->{lf} = {};
-  my $eps  = $opts{eps} // $prf->{eps} // 0; #0.5;
+  my $eps  = $opts{eps} // $prf->{eps} // 0.5; #0
   my ($i2,$f12);
   while (($i2,$f12)=each(%$pf12)) {
     $lf->{$i2} = log2($f12+$eps);
   }
   $prf->{score} = 'lf';
-  return $prf;
+  return $prf->compile_clean();
 }
 
 ## $prf = $prf->compile_lfm(%opts)
@@ -381,7 +392,7 @@ sub compile_lfm {
     $lfm->{$i2} = log2($f12+$eps) - $logM;
   }
   $prf->{score} = 'lfm';
-  return $prf;
+  return $prf->compile_clean();
 }
 
 ## $prf = $prf->compile_milf(%opts)
@@ -396,22 +407,21 @@ sub compile_milf {
   my ($prf,%opts) = @_;
   my ($N,$f1,$pf2,$pf12) = @$prf{qw(N f1 f2 f12)};
   my $mi = $prf->{milf} = {};
-  my $eps = $opts{eps} // $prf->{eps} // 0; #0.5;
-  my ($i2,$f2,$f12);
+  my $eps = $opts{eps} // $prf->{eps} // 0; #0.5
+  my ($i2,$f2,$f12,$denom);
   while (($i2,$f2)=each(%$pf2)) {
-    $f12 = $pf12->{$i2} // 0;
+    $f12   = ($pf12->{$i2} // 0) + $eps;
+    $denom = (($f1+$eps)*($f2+$eps));
     $mi->{$i2} = (
-		  log2($f12+$eps)
+		  ($f12 >= 0 ? log2($f12) : 0)
 		  *
-		  log2(
-		       (($f12+$eps)*($N+$eps))
-		       /
-		       (($f1+$eps)*($f2+$eps))
-		      )
+		  ($denom
+		   ? log2( (($f12+$eps)*($N+$eps)) / $denom )
+		   : 0)
 		 );
   }
   $prf->{score} = 'milf';
-  return $prf;
+  return $prf->compile_clean();
 }
 
 ## $prf = $prf->compile_mi1(%opts)
@@ -424,19 +434,17 @@ sub compile_mi1 {
   my ($N,$f1,$pf2,$pf12) = @$prf{qw(N f1 f2 f12)};
   my $mi = $prf->{mi1} = {};
   my $eps = $opts{eps} // $prf->{eps} // 0; #0.5;
-  my ($i2,$f2,$f12);
+  my ($i2,$f2,$f12,$denom);
   while (($i2,$f2)=each(%$pf2)) {
     $f12 = $pf12->{$i2} // 0;
-    $mi->{$i2} = (
-		  log2(
-		       (($f12+$eps)*($N+$eps))
-		       /
-		       (($f1+$eps)*($f2+$eps))
-		      )
+    $denom = (($f1+$eps)*($f2+$eps));
+    $mi->{$i2} = ($denom > 0
+		  ? log2( (($f12+$eps)*($N+$eps)) / $denom )
+		  : undef
 		 );
   }
   $prf->{score} = 'mi1';
-  return $prf;
+  return $prf->compile_clean();
 }
 
 
@@ -449,20 +457,18 @@ sub compile_mi3 {
   my ($prf,%opts) = @_;
   my ($N,$f1,$pf2,$pf12) = @$prf{qw(N f1 f2 f12)};
   my $mi3 = $prf->{mi3} = {};
-  my $eps = $opts{eps} // $prf->{eps} // 0; #0.5;
-  my ($i2,$f2,$f12);
+  my $eps = $opts{eps} // $prf->{eps} // 0; #0.5
+  my ($i2,$f2,$f12,$denom);
   while (($i2,$f2)=each(%$pf2)) {
-    $f12 = $pf12->{$i2} // 0;
-    $mi3->{$i2} = (
-		   log2(
-			(($f12+$eps)**3 * ($N+$eps))
-			/
-			(($f1+$eps)*($f2+$eps))
-		       )
+    $f12   = $pf12->{$i2} // 0;
+    $denom = (($f1+$eps)*($f2+$eps));
+    $mi3->{$i2} = ($denom
+		   ? log2( (($f12+$eps)**3 * ($N+$eps)) / $denom )
+		   : undef
 		  );
   }
   $prf->{score} = 'mi3';
-  return $prf;
+  return $prf->compile_clean();
 }
 
 ## $prf = $prf->compile_ld(%opts)
@@ -475,23 +481,30 @@ sub compile_ld {
   my ($N,$f1,$pf2,$pf12) = @$prf{qw(N f1 f2 f12)};
   my $ld = $prf->{ld} = {};
   my $eps = $opts{eps} // $prf->{eps} // 0; #0.5;
-  my ($i2,$f2,$f12);
+  my ($i2,$f2,$f12,$denom);
   while (($i2,$f2)=each(%$pf2)) {
-    $f12 = $pf12->{$i2} // 0;
-    $ld->{$i2} = 14 + log2(
-			   (2 * ($f12+$eps))
-			   /
-			   (($f1+$eps) + ($f2+$eps))
-			  );
+    $f12   = $pf12->{$i2} // 0;
+    $denom = ($f1+$eps) + ($f2+$eps);
+    $ld->{$i2} = ($denom
+		  ? (14 + log2( (2 * ($f12+$eps)) / $denom ))
+		  : undef);
   }
   $prf->{score} = 'ld';
-  return $prf;
+  return $prf->compile_clean();
 }
 
+## log0($x) : like log($x) but returns 0 for $x==0
 sub log0 {
+  no warnings 'uninitialized';
   return $_[0]>0 ? log($_[0]) : 0;
 }
 
+## log0d($num,$denom) : like log0($num/$denom), but returns 0 for $num==0 or $denom==0
+sub log0d {
+  no warnings 'uninitialized';
+  return $_[1]==0 ? 0 : log0($_[0]/$_[1]);
+}
+
 ## $prf = $prf->compile_ll(%opts)
 ##  + computes 1-sided log-log-likelihood ratio in $prf->{ll} a la Evert (2008)
 ##  + sets $pf->{score}='ll'
@@ -509,12 +522,14 @@ sub compile_ll {
   while (($i2,$f2)=each(%$pf2)) {
     $f12  = ($pf12->{$i2} // 0) + $eps;
     $logl = (##-- raw log-lambda
-	     $f12*log0($f12/($f1*$f2/$N))
-	     +($f1-$f12)*log0(($f1-$f12)/(($f1*($N-$f2)/$N)))
-	     +($f2-$f12)*log0(($f2-$f12)/(($N-$f1)*$f2/$N))
-	     +($N-$f1-$f2+$f12)*log0(($N-$f1-$f2+$f12)/(($N-$f1)*($N-$f2)/$N))
+	     $N<=0 ? 0
+	     : ($f12*log0d($f12, ($f1*$f2/$N))
+		+($f1-$f12)*log0d(($f1-$f12), (($f1*($N-$f2)/$N)))
+		+($f2-$f12)*log0d(($f2-$f12), (($N-$f1)*$f2/$N))
+		+($N-$f1-$f2+$f12)*log0d(($N-$f1-$f2+$f12), (($N-$f1)*($N-$f2)/$N))
+	       )
 	    );
-    $ll->{$i2} = (($f12 < ($f1*$f2/$N) ? -1 : 1) ##-- one-sided log-likelihood a la Evert (2008): negative for dis-associations
+    $ll->{$i2} = (($N && $f12 < ($f1*$f2/$N) ? -1 : 1) ##-- one-sided log-likelihood a la Evert (2008): negative for dis-associations
 		  #* $logl			 ##-- raw log-lambda values over-emphasize strong collocates
 		  * log0(1+$logl) 		 ##-- extra log() is better for scaling
 		  #* sqrt($logl)                 ##-- extra sqrt() for scaling
@@ -523,10 +538,11 @@ sub compile_ll {
   }
 
   $prf->{score} = 'll';
-  return $prf;
+  return $prf->compile_clean();
 }
 
 
+
 ##==============================================================================
 ## Trimming
 
