@@ -35,7 +35,7 @@ our %EXPORT_TAGS =
      run   => [qw(crun opencmd)],
      env   => [qw(env_set env_push env_pop)],
      pack  => [qw(packsize packsingle packFilterFetch packFilterStore)],
-     math  => [qw($LOG2 log2 min2 max2 lmax lmin lsum)],
+     math  => [qw(isNan isInf isFinite $LOG2 log2 min2 max2 lmax lmin lsum)],
      list  => [qw(luniq sluniq xluniq lcounts)],
      regex => [qw(regex)],
      html  => [qw(htmlesc)],
@@ -391,6 +391,19 @@ sub packFilterFetch {
 ##==============================================================================
 ## Math stuff
 
+sub isNan {
+  no warnings qw(uninitialized numeric);
+  return !($_[0]<=0||$_[0]>=0);
+}
+sub isInf {
+  no warnings qw(uninitialized numeric);
+  return !($_[0]<=0||$_[0]>=0) || ($_[0]==+"INF") || ($_[0]==-"INF");
+}
+sub isFinite {
+  no warnings qw(uninitialized numeric);
+  return ($_[0]<=0||$_[0]>=0) && ($_[0]!=+"INF") && ($_[0]!=-"INF");
+}
+
 our ($LOG2);
 BEGIN {
   $LOG2 = log(2.0);
